(* the description for retrieving a bitmap *)
type idx

(* operations to generate/load bitmaps *)
val char : char * Sdlvideo.color -> idx
val block : int * int * Sdlvideo.color * int -> idx
val file : string -> idx
val colored : Sdlvideo.color -> idx -> idx
val compose : idx list -> idx
val move : (int * int) -> idx -> idx
val realpha : float -> idx -> idx
val brightness : float -> idx -> idx
val combine : idx * idx * idx -> idx
val empty : idx
val tile_horiz : idx * int -> idx
val tile_vert : idx * int -> idx
val mirror_horiz : idx -> idx
val mirror_vert : idx -> idx
val no_cache : idx -> idx
val surface : Sdlvideo.surface -> idx
val surface_ofs : (int * int) * Sdlvideo.surface -> idx
val of_function : (int * int) * (int * int -> (int * int * int) * int) -> idx
val native : idx -> idx

(** a line originating from origo to the coordinates. ssslllooowwww.. also probably doesn't work. *)
val line : (int * int) -> idx

(** crop image ((x1, y1), (x2, y2)) *)
val crop : ((int * int) * (int * int)) -> idx -> idx

(** crop all invisible stuff *)
val auto_crop : idx -> idx

(* generate collision masks out of surfaces *)
(* mask consists of pixels that are non-zero *)
val collision_mask_of_surface : Sdlvideo.surface -> CollisionMask.mask
(* don't consider the alpha channel, consider the remaining pixels in inverse *)
val collision_mask_of_surface2 : Sdlvideo.surface -> CollisionMask.mask

(* the same, but for bitmaps *)
val mask_with_offset :
  (Sdlvideo.surface -> CollisionMask.mask) ->
  (int * int) * Sdlvideo.surface -> (int * int) * CollisionMask.mask

(* blit a bitmap (with offset information) *)
val blit : #Gfx.t -> int * int -> (int * int) * Sdlvideo.surface -> unit

(* blit an index *)
val blit_idx : #Gfx.t -> int * int -> idx -> unit

(* retrieve a bitmap *)
val bitmap : ?cache:bool -> idx -> (int * int) * Sdlvideo.surface

(* dump statistics *)
val dump_stats : unit -> unit
