open Graphics

class gfx = 
  let scale = ref 5 in
  let ofs_x, ofs_y = ref 400, ref 300 in
  let transform (x, y) = !scale * x + !ofs_x, 600 - (!ofs_y + !scale * y) in
object (_: 'self)
  val line_width = 1

  method move_to at = let (x, y) = transform at in moveto x y
  method line_to at = let (x, y) = transform at in lineto x y
  method set_color c = set_color c

  method with_width : 'a.int -> ('self -> 'a) -> 'a =
    fun w f ->
      set_line_width w;
      let v = GobaUtils.valuefy f {< line_width = w >} in
	set_line_width line_width;
	GobaUtils.pass v

  method horiz_ruler at = let (_, y) = transform (0, at) in 
    draw_segments [|(0, y, 800, y)|]

  method vert_ruler at = let (x, _) = transform (at, 0) in 
    draw_segments [|(x, 0, x, 600)|]

  method point at = let (x, y) = transform at in 
    draw_circle x y 3

  method set_offset (ox, oy) =
    ofs_x := ox;
    ofs_y := oy

  method set_scale s = scale := s
end

let dgfx = new gfx

let no_gfx : gfx =
object (self:'self)
  method move_to _ = ()
  method line_to _ = ()
  method set_color _ = ()
  method with_width : 'a.int -> (gfx -> 'a) -> 'a = fun _ f -> f self
  method horiz_ruler _ = ()
  method vert_ruler _ = ()
  method set_offset _ = ()
  method set_scale _ = ()
  method point _ = ()
end
