open VecOps

let polygon ?clip (gfx:Gfx.t) color pts =
  let pts = 
    match clip with
      | None -> pts
      | Some clip -> Polygon.clip_rect pts clip
  in
    Polygon.polygon 
      pts
      (fun row x0 x1 -> 
	 gfx#rectangle color (x0, row) (x1 - x0 + 1, 1))

(* primitive.. *)
let line (gfx:Gfx.t) color thickness (a, b) =
  let delta = Vector.float2 (b -| a) in
  let origo = Vector.float2 a in
  let len = Vector.abs2 delta in
  let step = delta /|. Vector.v2 len in
  let rec aux at =
    if at <= len then
      begin
	gfx#rectangle color (Vector.int2 (origo +|. step *|. Vector.v2 at)) (Vector.v2 thickness);
	aux (at +. 1.0)
      end
  in
    aux 0.0

let circle (gfx:Gfx.t) color thickness steps base radius =
  let step_len = GobaMath.pi *. 2.0 /. float steps in
  let rec aux at prev =
    if at <= GobaMath.pi *. 2.0 then
      begin
	let coord = base +| Vector.int2 (cos at *. radius, sin at *. radius) in
	  ( match prev with
	      | None -> ()
	      | Some prev_coord ->
		  line gfx color thickness (prev_coord, coord)
	  );
	aux (at +. step_len) (Some coord)
      end
  in
    aux 0.0 None

