open VecOps

let pi = 4.0 *. atan 1.0

let pi2 = 2.0 *. pi

let clamp low high v = min high (max low v)

let scale low high v =
  float (v - low) /. float (high - low)

let f_scale low high v =
  (v -. low) /. (high -. low)

let ipolate3 v col1 col2 =
  let v = clamp 0.0 1.0 v in
  let s = Vector.v3 v in
  let s'rev = Vector.v3 (1.0 -. v) in
    Vector.int3 (s'rev *||. Vector.float3 col1 +||. s *||. Vector.float3 col2)


let frame_of_radian direction =
  let deg = mod_float (((pi2 -. direction) /. pi2) *. 360.0) 360.0 in
  let deg = if deg < 0.0 then 360.0 +. deg else deg in
  let n = int_of_float (deg /. 10.0 +. 0.5) in
  let n = (n + 9) mod 36 in
    n

let abs_limit max v =
  if v > max then max
  else if v < ~-. max then ~-. max
  else v

let normalize_angle ang =
  let ang = mod_float ang pi2 in
    if ang < 0.0 then ang +. pi2 else ang

(* how much ang1 be changed to be the same as ang2, via the shortest rotation? *)
let angle_delta ang1 ang2 =
  let ang1 = normalize_angle ang1 in
  let ang2 = normalize_angle ang2 in
  let ang2 = if (ang2 < ang1) then ang2 +. pi2 else ang2 in
    (* at this point, ang2 is always bigger than ang1, and is no longer normalized *)
  let delta = ang2 -. ang1 in (* always positive *)
(*    Printf.printf "ang1=%f, ang2=%f, delta=%f\n%!" (ang1 /. pi) (ang2 /. pi) (delta /. pi);*)
    if delta < pi then
      (*let _ = Printf.printf "1\n" in*)
      delta
    else
      (*let _ = Printf.printf "2\n" in*)
	delta -. pi2
